@@ -135,12 +135,12 @@ class TestClient(unittest.TestCase):
     @patch('dns.resolver.query', dns_query)
     @patch('socket.getaddrinfo', socket_getaddrinfo)
     @patch('patroni.dcs.etcd.requests_get', requests_get)
+    @patch.object(EtcdClient, '_get_machines_list',
+                  Mock(return_value=['http://localhost:2379', 'http://localhost:4001']))
     def setUp(self):
-        with patch.object(EtcdClient, 'machines') as mock_machines:
-            mock_machines.__get__ = Mock(return_value=['http://localhost:2379', 'http://localhost:4001'])
-            self.client = EtcdClient({'srv': 'test', 'retry_timeout': 3}, DnsCachingResolver())
-            self.client.http.request = http_request
-            self.client.http.request_encode_body = http_request
+        self.client = EtcdClient({'srv': 'test', 'retry_timeout': 3}, DnsCachingResolver())
+        self.client.http.request = http_request
+        self.client.http.request_encode_body = http_request
 
     def test_machines(self):
         self.client._base_uri = 'http://localhost:4002'
@@ -156,9 +156,9 @@ class TestClient(unittest.TestCase):
         except Exception:
             self.assertIsNone(machines)
 
-    @patch.object(EtcdClient, 'machines')
-    def test_api_execute(self, mock_machines):
-        mock_machines.__get__ = Mock(return_value=['http://localhost:4001', 'http://localhost:2379'])
+    @patch.object(EtcdClient, '_get_machines_list',
+                  Mock(return_value=['http://localhost:4001', 'http://localhost:2379']))
+    def test_api_execute(self):
         self.client._base_uri = 'http://localhost:4001'
         self.assertRaises(etcd.EtcdException, self.client.api_execute, '/', 'POST', timeout=0)
         self.client._base_uri = 'http://localhost:4001'
@@ -196,11 +196,10 @@ class TestClient(unittest.TestCase):
     def test__get_machines_cache_from_dns(self):
         self.client._get_machines_cache_from_dns('error', 2379)
 
-    @patch.object(EtcdClient, 'machines')
-    def test__refresh_machines_cache(self, mock_machines):
-        mock_machines.__get__ = Mock(side_effect=etcd.EtcdConnectionFailed)
-        self.assertIsNone(self.client._refresh_machines_cache())
-        self.assertRaises(etcd.EtcdException, self.client._refresh_machines_cache, True)
+    @patch.object(EtcdClient, '_get_machines_list', Mock(side_effect=etcd.EtcdConnectionFailed))
+    def test__refresh_machines_cache(self):
+        self.assertFalse(self.client._refresh_machines_cache())
+        self.assertRaises(etcd.EtcdException, self.client._refresh_machines_cache, ['http://localhost:2379'])
 
     def test__load_machines_cache(self):
         self.client._config = {}
@@ -230,28 +229,27 @@ class TestClient(unittest.TestCase):
 class TestEtcd(unittest.TestCase):
 
     @patch('socket.getaddrinfo', socket_getaddrinfo)
+    @patch.object(EtcdClient, '_get_machines_list',
+                  Mock(return_value=['http://localhost:2379', 'http://localhost:4001']))
     def setUp(self):
-        with patch.object(EtcdClient, 'machines') as mock_machines:
-            mock_machines.__get__ = Mock(return_value=['http://localhost:2379', 'http://localhost:4001'])
-            self.etcd = Etcd({'namespace': '/patroni/', 'ttl': 30, 'retry_timeout': 10,
-                              'host': 'localhost:2379', 'scope': 'test', 'name': 'foo'})
+        self.etcd = Etcd({'namespace': '/patroni/', 'ttl': 30, 'retry_timeout': 10,
+                          'host': 'localhost:2379', 'scope': 'test', 'name': 'foo'})
 
     def test_base_path(self):
         self.assertEqual(self.etcd._base_path, '/patroni/test')
 
     @patch('dns.resolver.query', dns_query)
+    @patch('time.sleep', Mock(side_effect=SleepException))
+    @patch.object(EtcdClient, '_get_machines_list', Mock(side_effect=etcd.EtcdConnectionFailed))
     def test_get_etcd_client(self):
-        with patch('time.sleep', Mock(side_effect=SleepException)),\
-                patch.object(EtcdClient, 'machines') as mock_machines:
-            mock_machines.__get__ = Mock(side_effect=etcd.EtcdException)
-            self.assertRaises(SleepException, self.etcd.get_etcd_client,
-                              {'discovery_srv': 'test', 'retry_timeout': 10, 'cacert': '1', 'key': '1', 'cert': 1},
-                              EtcdClient)
-            self.assertRaises(SleepException, self.etcd.get_etcd_client,
-                              {'url': 'https://test:2379', 'retry_timeout': 10}, EtcdClient)
-            self.assertRaises(SleepException, self.etcd.get_etcd_client,
-                              {'hosts': 'foo:4001,bar', 'retry_timeout': 10}, EtcdClient)
-            mock_machines.__get__ = Mock(return_value=[])
+        self.assertRaises(SleepException, self.etcd.get_etcd_client,
+                          {'discovery_srv': 'test', 'retry_timeout': 10, 'cacert': '1', 'key': '1', 'cert': 1},
+                          EtcdClient)
+        self.assertRaises(SleepException, self.etcd.get_etcd_client,
+                          {'url': 'https://test:2379', 'retry_timeout': 10}, EtcdClient)
+        self.assertRaises(SleepException, self.etcd.get_etcd_client,
+                          {'hosts': 'foo:4001,bar', 'retry_timeout': 10}, EtcdClient)
+        with patch.object(EtcdClient, '_get_machines_list', Mock(return_value=[])):
             self.assertRaises(SleepException, self.etcd.get_etcd_client,
                               {'proxy': 'https://user:password@test:2379', 'retry_timeout': 10}, EtcdClient)
 
